﻿open System.Diagnostics

(* Raima.Rdm is the system module that contains the ADO.NET data provider
   for RDM *)
open Raima.Rdm

(* EXAMPLE - Perf06 for F# using ADO.NET *)
(* This is a simple performance example creates 50,000 rows in a simple table
 * in an on-disk database, using different transaction sizes (i.e. different
 * number of rows inserted as part of each transaction) to demonstrate the
 * performance difference when using small vs. large transactions.
 * 
 * The following methods are illustrated:
 *     Raima.Rdm.RdmConnection:
 *         ChangeDatabase(string) Method
 *         Close() Method
 *         CreateCommand() Method
 *         Open() Method
 *         RdmConnection(String) Constructor
 *     Raima.Rdm.RdmCommand:
 *         CommandText Property
 *         ExecuteNonQuery() Method
 *         ExecuteScalar() Method
 * 
 * They are fully document in http://docs.raima.com/rdm/.
 * 
 * For simplicity this example does not catch any thrown exceptions
 * but good programming practices would dictate that this occur in
 * a full application.
 *)

let NumRows = 50000

let DropDatabase (conn : RdmConnection) name =
    let cmd = conn.CreateCommand()

    (* Attempt to drop this database.  If the database does not already exist
     * this will fail and we simply ignore the failure for this sample. *)
    try
        cmd.CommandText <- "DROP DATABASE " + name
        cmd.ExecuteNonQuery() |> ignore
    with
        | exn -> ()

let CreateThisDatabase (conn : RdmConnection) name cmds =
    let cmd = conn.CreateCommand()

    cmd.CommandText <- "CREATE DATABASE " + name
    cmd.ExecuteNonQuery() |> ignore

    cmds |>
        Array.iter(fun c ->
            cmd.CommandText <- c
            cmd.ExecuteNonQuery() |> ignore
        )

    cmd.CommandText <- "COMMIT"
    cmd.ExecuteNonQuery() |> ignore

    cmd.CommandText <- "CLOSE " + name
    cmd.ExecuteNonQuery() |> ignore

let CreateDatabases conn =
    (* Drop any existing copies of the databases. *)
    DropDatabase conn "Perf06"

    (* Attempt to create the databases *)
    CreateThisDatabase conn "Perf06" [| "CREATE TABLE Simple (int_col INTEGER)" |]

(* Initialization - Open the connection and create the database *)
let Initialize (conn : RdmConnection) =
    let sw = Stopwatch.StartNew()

    conn.Open()
    CreateDatabases(conn)
    conn.ChangeDatabase("Perf06")
    
    sw.Stop()

    printfn "Initializing with DIRECT-LINK configuration"
    printfn "Preparing a new database:\t\t\t%d millisecs" sw.ElapsedMilliseconds
    printfn "\tOne, simple row type"
    printfn "\tOn disk"
    printfn "\tOpened exclusively"

(* Cleanup - Close the connection *)
let Cleanup (conn : RdmConnection) = 
    let sw = Stopwatch.StartNew()
    conn.Close()
    sw.Stop()

    printfn "Cleanup:\t\t\t\t\t%d millisecs" sw.ElapsedMilliseconds

(* Add 50,000 rows - there is no 'quick' method of inserting rows in
 * bulk in SQL, so we have to execute the INSERT command 50,000 times.
 * When adding rows, break them into transactions of 'transactionSize'
 * rows per transaction. *)
let AddRows (conn : RdmConnection) transactionSize =
    printf "Add %d rows with %-5d rows/transaction" NumRows transactionSize

    let sw = Stopwatch.StartNew()

    let cmd = conn.CreateCommand()
    cmd.CommandText <- "INSERT INTO Simple VALUES 1"

    (* Insert each row.  Every 'transactionSize' rows commit the current 
     * transaction.  The current transaction and count is threaded through
     * the operation as a tuple in the accumulator. *)
    let (trans, _) =
        seq { 1 .. NumRows } |>
            Seq.fold (fun (trans, count) ii ->
                let newtrans =
                    match trans with
                    | None -> conn.BeginTransaction()
                    | Some(t) -> t

                cmd.ExecuteNonQuery() |> ignore

                match count + 1 with
                | z when z = transactionSize ->
                    newtrans.Commit()
                    (None, 0)
                | z -> (Some(newtrans), z)
            ) (None, 0)

    (* If a transaction is still active, commit it *)
    match trans with
    | None -> ()
    | Some(t) -> t.Commit()

    sw.Stop()

    printfn "\t%d millisecs" sw.ElapsedMilliseconds

(* Delete all rows *)
let DeleteRows (conn : RdmConnection) =
    let sw = Stopwatch.StartNew()

    let trans = conn.BeginTransaction()

    let cmd = conn.CreateCommand()
    cmd.CommandText <- "DELETE FROM Simple"
    cmd.ExecuteNonQuery() |> ignore

    trans.Commit()

    sw.Stop()

printfn "Perf06 Example"
printfn ""

(* Create the connection with a simple connection string *)
let conn = new RdmConnection("host=local;openmode=x;tfstype=tfst;autocommit=off")

(* Open the connection, delete any existing database, and create the database
 * anew *)
Initialize conn

printfn ""

[ 50000; 10000; 5000; 1000; 500; 100 ] |>
    List.iter(fun transSize ->
        (* Add 50,000 rows to a table *)
        AddRows conn transSize

        (* Delete all the rows in the table *)
        DeleteRows conn
    )

printfn ""

(* Cleanup *)
Cleanup conn

printfn ""
printfn "Hit <Enter> to continue"
System.Console.ReadLine() |> ignore

